﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.Mvc;
using Kreta.BusinessLogic.Classes;
using Kreta.BusinessLogic.HelperClasses;
using Kreta.BusinessLogic.HelperClasses.ImportCo;
using Kreta.BusinessLogic.Helpers.ImportExport;
using Kreta.BusinessLogic.Security;
using Kreta.Enums;
using Kreta.Enums.ManualEnums;
using Kreta.Enums.ManualEnums.ImportExport;
using Kreta.Resources;
using Kreta.Web.Areas.ImportExport.Models.CommonModels;
using Kreta.Web.Areas.ImportExport.Models.TeremImportExport;
using Kreta.Web.Classes;
using Kreta.Web.Helpers;
using Kreta.Web.Models.EditorTemplates;
using Kreta.Web.Security;

namespace Kreta.Web.Areas.ImportExport.Controllers
{
    [MvcRoleClaimsAuthorize(true)]
    [MvcRolePackageDenyAuthorize(KretaClaimPackages.IsOnlyAlkalmozott.ClaimValue)]
    [MvcRolePackageAuthorize(TanevEnum.AktEsKovTanev, KretaClaimPackages.Adminisztrator.ClaimValue)]
    public class TeremImportExportController : BaseImportExportController
    {
        #region Fields

        private const string TovabbUrl = @"~/Intezmeny/Terem";

        #endregion Fields

        #region Properties

        public static string GridName => "TeremGrid";
        public static string GridRowTemplateName => "TeremGridRowTemplate";

        public static string JavascriptHelperClassName => "TeremImportExportHelper";

        public static string ConrtollerName => "TeremImportExport";

        #endregion Properties

        #region Constructors

        public TeremImportExportController(IUploadFileValidator uploadFileValidator)
        {
            UploadFileValidator = uploadFileValidator ?? throw new ArgumentNullException(nameof(uploadFileValidator));
        }

        #endregion Constructors

        public ActionResult Index()
        {
            var model = new TeremImportExportModel
            {
                MaxAllowedFileSizeInBytes = UploadFileValidator.Configuration.ImportMaxAllowedFileSizeInBytes,
                AllowedFileExtensions = UploadFileValidator.Configuration.ImportAllowedFileExtensions
            };
            return View(model);
        }

        #region Sablon

        public ActionResult DownloadTemplate()
        {
            MemoryStream memoryStream = new TeremImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetTemplate();
            return new FileStreamResult(memoryStream, Core.Constants.ContentTypes.Xlsx) { FileDownloadName = ImportExportTeremResource.ImportSablonFileName };
        }

        #endregion Sablon

        #region Export

        [HttpPost]
        [MvcValidateAjaxAntiForgeryToken]
        public string Export()
        {
            MemoryStream memoryStream = new TeremImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetExport(false);
            return GetExportFileString(memoryStream);
        }

        [HttpPost]
        [MvcValidateAjaxAntiForgeryToken]
        public string ExportTermekAdatai()
        {
            MemoryStream memoryStream = new TeremImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetExport(true);
            return GetExportFileString(memoryStream);
        }

        public string NemImportalhatoSorokExport()
        {
            MemoryStream memoryStream = new TeremImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).GetNemImportalhatoSorokExport();
            return GetExportFileString(memoryStream);
        }

        #endregion Export

        #region Import

        public JsonResult UploadFile(HttpPostedFileBase importFile, int importMuvelet, int? mukodesiHelyId)
        {
            UploadFileValidator.ValidateImportFile(importFile);

            var errorResult = new List<string>();
            if (mukodesiHelyId == null)
            {
                errorResult.Add(ErrorResource.MukodesiHelyMegadasaKotelezo);
            }

            Dictionary<int, string> importHeaderList = TeremImportExportHelper.ImportHeaderList;
            List<List<string>> importData = ImportExportHelper.GetImportDataFromExcel(importFile.InputStream, importHeaderList, out List<string> fileErrorResultList);
            errorResult.AddRange(fileErrorResultList);

            if (errorResult.Count > 0)
            {
                JsonResult errorJson = ThrowCustomError(errorResult);
                return errorJson;
            }

            //NOTE: mukodesiHelyId.Value nem lehet null a korábbi vizsgálatok miatt!
            new TeremImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).SetImportCo(importData, importMuvelet, mukodesiHelyId.Value);

            return Json(null);
        }

        public ActionResult OpenImportPreviewPopup()
        {
            TeremImportCo importCo = new TeremImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).ImportCo;
            List<TeremItemCo> coList = importCo.TeremCoList;

            var importInfoModel = new TeremImportInfoModel
            {
                ImportInfoSummaryList = new ImportInfoSummaryListModel
                {
                    ImportInfoSummaryModelList = new List<ImportInfoSummaryModel>
                    {
                        GetImportInfoSummary(importCo, coList)
                    }
                },
                MainImportItemList = new TeremImportItemListModel
                {
                    ImportItemModelList = GetImportItemModelList(importCo, coList)
                },
                NemImportalhatoItemList = new TeremImportNemImportalhatoItemListModel
                {
                    NemImportalhatoItemModelList = GetNemImportalhatoItemModelList(importCo.NemImportalhatoItemList)
                }
            };

            var popUpModel = new PopUpModel(importInfoModel, ImportPreviewPopupName);
            popUpModel.AddCancelBtn(popUpModel, "KretaWindowHelper.destroyAllWindow");
            popUpModel.AddBtn(popUpModel, "ImportBtnOk", CommonResource.Importalas, $"{JavascriptHelperClassName}.import", "BtnOk", setDisabledAfterClick: true);
            popUpModel.AddBtn(popUpModel, "NemImportalhatoSorokExportBtnOk", ImportExportCommonResource.NemImportalhatoSorokExportalasa, $"{JavascriptHelperClassName}.nemImportalhatoSorokExport", "BtnOk");
            return PartialView(Constants.General.PopupView, popUpModel);
        }

        private static ImportInfoSummaryModel GetImportInfoSummary(TeremImportCo importCo, List<TeremItemCo> coList)
        {
            List<TeremImportJsonItemCo> mainImportJsonItemList = importCo.MainImportJsonItemList;
            int insertRowCount = mainImportJsonItemList?.Count(x => x.Operation == (int)ImportItemOperationEnum.Insert) ?? 0;
            int updateRowCount = mainImportJsonItemList?.Count(x => x.Operation == (int)ImportItemOperationEnum.Update) ?? 0;
            int deleteRowCount = importCo.ImportMuvelet == (int)ImportMuveletEnum.DeleteAndInsert ? coList.Count(x => x.Importalt) : 0;
            int unchangedRowCount;
            switch (importCo.ImportMuvelet)
            {
                case (int)ImportMuveletEnum.DeleteAndInsert:
                    unchangedRowCount = coList.Count(x => !x.Importalt);
                    break;

                case (int)ImportMuveletEnum.InsertAndUpdate:
                    unchangedRowCount = coList.Count - updateRowCount;
                    break;

                default:
                    unchangedRowCount = coList.Count;
                    break;
            }

            int invalidRowCount = importCo.NemImportalhatoItemList.Count;
            var importInfoSummary = new ImportInfoSummaryModel
            {
                ImportName = ImportExportTeremResource.ImportDefaultSheetName,
                InsertRowCount = insertRowCount.ToString(),
                UpdateRowCount = updateRowCount.ToString(),
                DeleteRowCount = deleteRowCount.ToString(),
                UnchangedRowCount = unchangedRowCount.ToString(),
                InvalidRowCount = invalidRowCount.ToString()
            };
            return importInfoSummary;
        }

        private List<TeremImportItemModel> GetImportItemModelList(TeremImportCo importCo, List<TeremItemCo> coList)
        {
            var importItemModelList = new List<TeremImportItemModel>();
            List<TeremImportJsonItemCo> mainImportJsonItemList = importCo.MainImportJsonItemList;
            if (mainImportJsonItemList == null)
            {
                if (!(coList.Count > 0))
                {
                    return importItemModelList;
                }

                mainImportJsonItemList = new List<TeremImportJsonItemCo>();
            }

            //NOTE: Hozzáadjuk azokat a az elemeket, amik az excel-ből jöttek!
            foreach (TeremImportJsonItemCo importJsonItem in mainImportJsonItemList)
            {
                importItemModelList.Add(new TeremImportItemModel
                {
                    Nev = importJsonItem.Nev,
                    JellegNev = importJsonItem.JellegId.GetDisplayName<TeremTipusEnum>(ClaimData.SelectedTanevID.Value),
                    Kapacitas = importJsonItem.Kapacitas != null ? importJsonItem.Kapacitas.ToString() : string.Empty,
                    Terulet = importJsonItem.Terulet != null ? importJsonItem.Terulet.ToString() : string.Empty,
                    Operation = importJsonItem.Operation
                });
            }

            //NOTE: Hozzáadjuk azokat a az elemeket, amik az adatbázisból jöttek!
            foreach (TeremItemCo co in coList)
            {
                if (!mainImportJsonItemList.Select(x => x.Id).Contains(co.Id))
                {
                    importItemModelList.Add(new TeremImportItemModel
                    {
                        Nev = co.Nev,
                        JellegNev = co.JellegNev,
                        Kapacitas = co.Kapacitas != null ? co.Kapacitas.ToString() : string.Empty,
                        Terulet = co.Terulet != null ? co.Terulet.ToString() : string.Empty,
                        Operation = importCo.ImportMuvelet == (int)ImportMuveletEnum.DeleteAndInsert && co.Importalt ? (int)ImportItemOperationEnum.Delete : (int)ImportItemOperationEnum.Default
                    });
                }
            }

            return importItemModelList.OrderBy(x => x.Nev).ThenByDescending(x => x.OperationText).ToList();
        }

        private List<TeremImportNemImportalhatoItemModel> GetNemImportalhatoItemModelList(List<TeremImportItemCo> nemImportalhatoItemList)
        {
            var nemImportalhatoItemModelList = new List<TeremImportNemImportalhatoItemModel>();
            foreach (TeremImportItemCo nemImportalhatoItem in nemImportalhatoItemList)
            {
                nemImportalhatoItemModelList.Add(new TeremImportNemImportalhatoItemModel
                {
                    HelyisegNevImportData = nemImportalhatoItem.HelyisegNevImportData,
                    JellegImportData = nemImportalhatoItem.JellegImportData,
                    BefogadokepessegImportData = nemImportalhatoItem.BefogadokepessegImportData,
                    AlapteruletImportData = nemImportalhatoItem.AlapteruletImportData,
                    ErrorList = string.Join(Environment.NewLine, nemImportalhatoItem.ErrorList)
                });
            }

            return nemImportalhatoItemModelList.OrderBy(x => x.HelyisegNevImportData).ToList();
        }

        public ActionResult Import()
        {
            if (ModelState.IsValid)
            {
                TeremImportCo importCo = new TeremImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).ImportCo;
                new ImportExportHelper(ConnectionTypeExtensions.GetSessionConnectionType()).Import(new { ImportJson = importCo }, importCo.ImportMuvelet);
                return Content(Url.Content(TovabbUrl));
            }

            return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
        }

        #endregion Import
    }
}
